@@ -33,6 +33,7 @@ import (
 
 	"github.com/AlecAivazis/survey/v2"
 	"github.com/AlecAivazis/survey/v2/terminal"
+	"github.com/MakeNowJust/heredoc/v2"
 	"github.com/alecthomas/chroma/v2/quick"
 	"github.com/apex/log"
 	"github.com/blacktop/go-plist"
@@ -80,16 +81,21 @@ func init() {
 var entCmd = &cobra.Command{
 	Use:   "ent",
 	Short: "Search IPSW filesystem DMG or Folder for MachOs with a given entitlement",
-	Example: `  # Search IPSW for entitlement key
-  ❯ ipsw ent --ipsw <IPSW> --db /tmp --key platform-application
-  # Search local folder for entitlement key
-  ❯ ipsw ent --input /usr/bin --db /tmp --val platform-application
-  # Search IPSW for entitlement value (i.e. one of the <array> strings)
-  ❯ ipsw ent --ipsw <IPSW> --db /tmp --val LockdownMode
-  # Dump entitlements for MachO in IPSW
-  ❯ ipsw ent --ipsw <IPSW> --db /tmp --file WebContent
-  # Diff two IPSWs
-  ❯ ipsw ent --diff --ipsw <PREV_IPSW> --ipsw <NEW_IPSW> --db /tmp`,
+	Example: heredoc.Doc(`
+		# Search IPSW for entitlement key
+		❯ ipsw ent --ipsw <IPSW> --db /tmp --key platform-application
+
+		# Search local folder for entitlement key
+		❯ ipsw ent --input /usr/bin --db /tmp --val platform-application
+
+		# Search IPSW for entitlement value (i.e. one of the <array> strings)
+		❯ ipsw ent --ipsw <IPSW> --db /tmp --val LockdownMode
+
+		# Dump entitlements for MachO in IPSW
+		❯ ipsw ent --ipsw <IPSW> --db /tmp --file WebContent
+
+		# Diff two IPSWs
+		❯ ipsw ent --diff --ipsw <PREV_IPSW> --ipsw <NEW_IPSW> --db /tmp`),
 	Args:          cobra.NoArgs,
 	SilenceErrors: true,
 	RunE: func(cmd *cobra.Command, args []string) error {
